"""
Values preservation
===================

A `dolmen.content` content type can provide values described in the
schema at the class level. These values are thus preserved::

  >>> harfagri = Ynglingar()
  >>> harfagri.rank
  u'Jarl'

  >>> gormsson = JomsWarrior()
  >>> gormsson.rank
  u'Bondi'

"""

import dolmen.content
from zope.interface import Interface
from zope.schema import Choice


class IViking(Interface):
    """Defines a Norseman
    """
    rank = Choice(
        title=u"Rank of the viking warrior",
        default=u"Jarl",
        values=[u"Bondi", u"Hersir", u"Jarl", u"Einherjar"])


class Ynglingar(dolmen.content.Content):
    dolmen.content.schema(IViking)


class JomsWarrior(dolmen.content.Content):
    dolmen.content.schema(IViking)
    rank = u"Bondi"



# -*- coding: utf-8 -*-

import unittest
import doctest
import pkg_resources
from dolmen.content import tests


def make_test(dottedname):
    test = doctest.DocTestSuite(
        dottedname, setUp=tests.siteSetUp, tearDown=tests.siteTearDown,
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    test.layer = tests.DolmenContentLayer(tests)
    return test


def suiteFromPackage(name):
    files = pkg_resources.resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'dolmen.content.tests.%s.%s' % (name, filename[:-3])
        suite.addTest(make_test(dottedname))
    return suite


def test_suite():
    suite = unittest.TestSuite()
    readme = doctest.DocFileSuite(
        '../README.txt', globs={'__name__': 'dolmen.content'},
        optionflags=(doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS))
    readme.layer = tests.DolmenContentLayer(tests)
    suite.addTest(readme)
    for name in ['overview', 'schema', 'factory']:
        suite.addTest(suiteFromPackage(name))
    return suite



"""
Custom factories
================

Explicitly linked factories are registered normally::

  >>> from zope.component import getUtility, queryUtility
  >>> joe = getUtility(dolmen.IFactory, 'baker_joe')
  >>> isinstance(joe, BakerJoe)
  True

  >>> joe.title
  u"Joe's bakery"
  >>> joe.description
  u'I am a totally custom factory.'

Factories that are not linked with the `factory` directive or by
natural association are disregarded::

  >>> steve = queryUtility(dolmen.IFactory, 'baker_steve')
  >>> steve is None
  True

"""

import dolmen.content as dolmen
from zope.interface import Interface, implements


class IBread(Interface):
    pass


class BakerJoe(dolmen.Factory):
    dolmen.name('baker_joe')
    title = u"Joe's bakery"
    description = u"I am a totally custom factory."


class BakerSteve(dolmen.Factory):
    dolmen.name('baker_steve')
    title = u"Steve and Sons."


class Baguette(dolmen.Content):
    """A crusty bread.
    """
    dolmen.name(u'White fresh bread')
    dolmen.factory(BakerJoe)
    implements(IBread)



"""
Values initialization
=====================

A `dolmen.content` content type is defined by its schema. When a schema
is applied on a content type, the fields are bootstrapped on the class,
and the instance of the class uses the default values.

  >>> mcDuff = Thane()
  >>> mcDuff.age
  0
  >>> mcDuff.thanehood
  []

It's possible, however, to give initial values to the constructor::

  >>> mcBeth = Thane(age=35, thanehood=[u'Glamis', u'Cawdor'])
  >>> mcBeth.age
  35
  >>> mcBeth.thanehood
  [u'Glamis', u'Cawdor']

The values are verified according to the schema. Elements that are not
part of the schema are currently skipped::

Providing a wrong type or unaccepted value will raise an error::

  >>> lennox = Thane(age='45')
  Traceback (most recent call last):
  ...
  WrongType: ('45', (<type 'int'>, <type 'long'>), 'age')

Providing a non-existing field value will do nothing::

  >>> lady = Thane(madness=True)
  >>> print lady
  <dolmen.content.tests.schema.initialize.Thane object at ...>

"""
import dolmen.content
from zope.interface import Interface
from zope.schema import Int, List, TextLine


class IThane(Interface):
    """Defines a Scotland's Thane
    """
    age = Int(
        title=u"Age of the Thane",
        default=0)

    thanehood = List(
        title=u"Names of the thane's domains",
        value_type=TextLine(),
        default=[],
        required=True)


class Thane(dolmen.content.Content):
    dolmen.content.schema(IThane)



# -*- coding: utf-8 -*-

from grokcore.component import name, title, description
from grokcore.security import require

from dolmen.content.interfaces import *
from dolmen.content.directives import schema, factory, nofactory
from dolmen.content.base import Content, Container, OrderedContainer
from dolmen.content.utils import get_content_type
from dolmen.content.factoring import Factory



"""
Content type
============

A ``dolmen.content`` item, when grokked, gets a `__content_type__`
attributes, that can be useful to sort/classify/index the objects.

This `__content_type__` is computed from the `name` directive or the
class name, if no `name` directive is used::

  >>> from dolmen.content import Content
  >>> from grokcore.component import testing


No directive
------------

  >>> class SomeContent(Content):
  ...    pass

  >>> SomeContent.__content_type__
  Traceback (most recent call last):
  ...
  AttributeError: type object 'SomeContent' has no attribute
  '__content_type__'

  >>> testing.grok_component('somecontent', SomeContent)
  True

  >>> print SomeContent.__content_type__
  SomeContent


Directive
---------

  >>> from dolmen.content import name

  >>> class AnotherContent(Content):
  ...   name('my nice content')

  >>> AnotherContent.__content_type__
  Traceback (most recent call last):
  ...
  AttributeError: type object 'AnotherContent' has no attribute
  '__content_type__'

  >>> testing.grok_component('another', AnotherContent)
  True

  >>> print AnotherContent.__content_type__
  my nice content


Heritage
--------

  >>> class YetAnotherContent(AnotherContent):
  ...   name('I do override the parent value')

  >>> print YetAnotherContent.__content_type__
  my nice content

  >>> testing.grok_component('yetanother', YetAnotherContent)
  True

  >>> print YetAnotherContent.__content_type__
  I do override the parent value


Utility function
----------------

  >>> from dolmen.content import get_content_type

  >>> print get_content_type(YetAnotherContent)
  I do override the parent value

  >>> obj = AnotherContent()
  >>> print get_content_type(obj)
  my nice content

  >>> toto = object()
  >>> print get_content_type(toto)
  None

"""



# -*- coding: utf-8 -*-

import martian
import warnings
import dolmen.content
import grokcore.security
import grokcore.component
from martian.util import scan_for_classes
from zope.component import provideUtility
from zope.interface import verify


class FactoryGrokker(martian.GlobalGrokker):
    """Grokker dedicated to find the factory within a module.
    """
    martian.priority(1002)

    def get_default(cls, component, module=None, **data):
        components = list(scan_for_classes(module, dolmen.content.IFactory))
        if len(components) == 0:
            return None
        elif len(components) == 1:
            component = components[0]
        else:
            return None
        return component

    def grok(self, name, module, module_info, config, **kw):
        factory = self.get_default(module, module)
        if factory is not None:
            dolmen.content.factory.set(module, factory)
        return True


class ContentTypeGrokker(martian.ClassGrokker):
    martian.component(dolmen.content.Content)
    martian.directive(dolmen.content.factory)
    martian.directive(grokcore.component.name,
                      get_default=dolmen.content.factoring.default_name)
    martian.directive(grokcore.security.require)

    def grok(self, name, content, module_info, **kw):
        content.module_info = module_info
        return martian.ClassGrokker.grok(
            self, name, content, module_info, **kw)

    def execute(self, content, config, name, factory, require, **kw):

        if not isinstance(name, unicode):
            try:
                name = unicode(name)
            except:
                raise martian.error.GrokError(
                    "%r is not a unicode value. Content `name`"
                    " should be a unicode string." % name)

        content.__content_type__ = name

        if dolmen.content.nofactory.bind().get(content):
            if factory:
                warnings.warn(
                    ("Your Content type has an explicit Factory '%s'."
                     " At the same time you specified the *nofactory*"
                     " directive for your Content type '%s'. The"
                     " factory will be ignored.") %
                     (factory.__name__, content.__name__), UserWarning, 2)
            return True

        elif factory is None:
            utility = dolmen.content.Factory(content)
            verify.verifyObject(dolmen.content.IFactory, utility, tentative=0)
            factory_name = '%s.%s' % (content.__module__, content.__name__)

        else:
            factory_name = grokcore.component.name.bind().get(factory)
            if not factory_name:
                raise martian.error.GrokError(
                    "%r is used as a contenttype factory by %r. "
                    "However, the factory name was omitted. Please, "
                    "use the `name` directive to define a factory name."
                    % (factory, content), factory)
            utility = factory(content)

        config.action(
            discriminator=('utility', dolmen.content.IFactory, factory_name),
            callable=provideUtility,
            args=(utility, dolmen.content.IFactory, factory_name),
            )

        return True



# -*- coding: utf-8 -*-

from dolmen.content import schema, name, title, description
from dolmen.content.interfaces import IFactory
from zope.schema.fieldproperty import FieldProperty
from zope.interface import implements, implementedBy


def default_name(component, module=None, **data):
    return component.__name__


class Factory(object):
    """A content type generic factory.
    """
    implements(IFactory)

    addform = FieldProperty(IFactory['addform'])

    def __init__(self, factory):
        self.factory = factory

    def getInterfaces(self):
        return implementedBy(self.factory)

    def getSchema(self):
        return schema.bind().get(self.factory)

    @property
    def name(self):
        return name.bind(get_default=default_name).get(self.factory)

    @property
    def title(self):
        return title.bind(default=u"").get(self.factory)

    @property
    def description(self):
        return description.bind(default=u"").get(self.factory)

    def __call__(self, *args, **kw):
        return self.factory(*args, **kw)



"""
Instanciation
=============

A dolmen.content Content only needs a name to be fully useable.

  >>> mongo = Dummy()
  >>> mongo.__content_type__
  u'DummyContent'

  >>> from grokcore.component.interfaces import IContext
  >>> IContext.providedBy(mongo)
  True


Schema
======

This content is getting a base interface and a base schema.
It means that the attributes are set by default on the content type.
Of course, the basic fields validations are respected.

  >>> IDummySchema.providedBy(mongo)
  True
  >>> mongo.title
  u''
  >>> mongo.title = 'a vast bowl of pus'
  Traceback (most recent call last):
  ...
  WrongType: ('a vast bowl of pus', <type 'unicode'>, 'title')

  >>> mongo.title = u'Oh... it makes me mad... mad!'
  >>> mongo.title
  u'Oh... it makes me mad... mad!'


Factory
=======

In order to get an abstract and generic way to handle the content types,
`dolmen.content` provides a factory system that permits to instanciate your
objects.

Further, if no factory is explicitly declared, one is automatically generated
and registered, using the package path and class name as an identifier.

  >>> from dolmen.content import IFactory
  >>> from zope.component import getUtility
  >>> myfactory = getUtility(IFactory,
  ...                        name='dolmen.content.tests.overview.simple.Dummy')

  >>> myfactory.factory
  <class 'dolmen.content.tests.overview.simple.Dummy'>

  >>> myfactory.getSchema()
  [<InterfaceClass dolmen.content.tests.overview.simple.IDummySchema>]

The factory describes the generated content::

  >>> myfactory.name
  'DummyContent'

  >>> myfactory.title
  'A Dummy content'

  >>> myfactory.description
  'A very dumb and dull content.'

Finally, the factory provides the name of the view serving as an add
form to add the content through the web::

  >>> myfactory.addform
  u'dolmen.add'

  >>> from zope.interface.verify import verifyObject
  >>> verifyObject(IFactory, myfactory)
  True

Content type with no directives
-------------------------------

  >>> infoless = getUtility(IFactory,
  ...              name='dolmen.content.tests.overview.simple.InfoLess')

  >>> infoless.name
  'InfoLess'

  >>> infoless.description
  u''

  >>> infoless.title
  u''

  >>> infoless.addform
  u'dolmen.add'

  >>> verifyObject(IFactory, infoless)
  True

"""

import dolmen.content as dolmen
from zope.schema import TextLine
from zope.interface import Interface


class IDummySchema(Interface):
    title = TextLine(
        title=u"Title",
        required=True,
        default=u'')


class Dummy(dolmen.Content):
    """A very simple content
    """
    dolmen.name('DummyContent')
    dolmen.title('A Dummy content')
    dolmen.description('A very dumb and dull content.')
    dolmen.schema(IDummySchema)


class InfoLess(dolmen.Content):
    pass



"""
Content location
================

A ``dolmen.content`` content is locatable. It means it can belong to a
hierarchy and, therefore, hold informations about its current position
or location.

  >>> from zope.location import ILocation
 
Let's work with a content and a container::

  >>> cave = Cave()
  >>> moody = Tiger()

  >>> ILocation.providedBy(cave)
  True

  >>> ILocation.providedBy(moody)
  True

As we can see, out-of-the-box, ``dolmen.content`` contents provide
ILocation, meaning they have a `__parent__` and `__name__`::

  >>> from zope.interface.verify import verifyObject

  >>> verifyObject(ILocation, cave)
  True

  >>> verifyObject(ILocation, moody)
  True

While working with containment, these attributes will be most
important::

  >>> cave['ugly cat'] = moody

  >>> moody.__name__
  u'ugly cat'
  
  >>> moody.__parent__
  <dolmen.content.tests.schema.location.Cave object at ...>

While deleted from the container, the attributes will be set to None::

  >>> del cave['ugly cat']

  >>> print moody.__name__
  None

  >>> print moody.__parent__
  None

"""

import dolmen.content as dolmen


class Cave(dolmen.Container):
    """A rocky shelter
    """
    dolmen.name('Cave')


class Tiger(dolmen.Content):
    """A wild animal
    """
    dolmen.name('Sabertooth Tiger')



# -*- coding: utf-8 -*-

import grokcore.content
from dolmen.content import interfaces, require
from dolmen.content.directives import schema, Fields
from grokcore.component import baseclass
from zope.interface import implements


class Content(grokcore.content.Model):
    baseclass()
    require("zope.ManageContent")
    implements(interfaces.IContent)

    def __init__(self, **kwargs):
        grokcore.content.Model.__init__(self)
        if kwargs:
            schemas = schema.bind().get(self)
            if schemas:
                ifields = Fields(*schemas)
                for key, value in kwargs.items():
                    ifield = ifields.get(key)
                    if ifield is None:
                        continue
                    field = ifield.bind(self)
                    field.validate(value)
                    field.set(self, value)


class Container(Content, grokcore.content.Container):
    """A dolmen folderish content type.
    """
    baseclass()
    implements(interfaces.IContainer)

    def __init__(self, **kwargs):
        grokcore.content.Container.__init__(self)
        Content.__init__(self, **kwargs)


class OrderedContainer(Content, grokcore.content.OrderedContainer):
    """A dolmen folderish content type with ordered keys.
    """
    baseclass()
    implements(interfaces.IOrderedContainer)

    def __init__(self, **kwargs):
        grokcore.content.OrderedContainer.__init__(self)
        Content.__init__(self, **kwargs)



"""
Multiple schemas example
========================

Sonja The Red is a famous Hero and a Barbarian. She has a certain killing
standard but, all things considered, she's not a savage and she likes fancy
outfits. Two schemas can summarize her killing and fashion tendencies :

  >>> sonja = FemaleBarbarian()
  >>> IBarbarian.providedBy(sonja)
  True
  >>> IFemaleHero.providedBy(sonja)
  True

Sonja is maybe a barbarian, but her Redhair is one of her most priced assets.
So, if defined at the class level, the attribute will not be overriden and
the given value will be used.

  >>> sonja.nickname
  u'The Red'

The other attributes of our hero are still set to default.

  >>> sonja.armor
  u'metal bikini'
  
"""
import dolmen.content as dolmen
from zope.interface import Interface
from zope.schema import TextLine, Int, Choice


class IFemaleHero(Interface):
    """A female hero, noticeable by her armor.
    """
    armor = Choice(
        title=u"Armor",
        values=[u'metal bikini', u'leather bikini'],
        default=u'metal bikini')

    
class IBarbarian(Interface):
    """A barbarian. Usually only wearing a leather underpants.
    """
    kills = Int(
        title=u"Kills !",
        default=100)

    nickname = TextLine(
        title=u"Nickname",
        default=u"The Barbarian")


class FemaleBarbarian(dolmen.Content):
    """A sexy barbarian content
    """
    dolmen.name('female')
    dolmen.schema(IFemaleHero, IBarbarian)
    nickname = u"The Red"



"""
Base components
===============

This test's purpose is to test simple components instanciation
after grokking.

We first declare our components and grok them::

  >>> import dolmen.content
  >>> from zope.schema import TextLine
  >>> from zope.interface import Interface
  >>> from grokcore.component.testing import grok_component

  >>> class TitleSchema(Interface):
  ...     title = TextLine(
  ...       title=u'Title',
  ...       required=True,
  ...       default=u'')

  >>> class Mammoth(dolmen.content.Content):
  ...   dolmen.content.name('mammoth')
  ...   dolmen.content.schema(TitleSchema)

  >>> grok_component('mammoth', Mammoth)
  True

  >>> class Cave(dolmen.content.Container):
  ...   dolmen.content.name('cave')
  ...   dolmen.content.schema(TitleSchema)

  >>> grok_component('cave', Cave)
  True

  >>> class HunterQuiver(dolmen.content.OrderedContainer):
  ...   dolmen.content.name('quiver')
  ...   dolmen.content.schema(TitleSchema)

  >>> grok_component('quiver', HunterQuiver)
  True

We now instanciate them with and without arguments::

  >>> gunther = Mammoth()
  >>> gunther
  <dolmen.content.tests.overview.base.Mammoth object at ...>

  >>> manfred = Mammoth(title=u'Manfred the Impetuous')
  >>> manfred
  <dolmen.content.tests.overview.base.Mammoth object at ...>
  >>> manfred.title
  u'Manfred the Impetuous'

  >>> grotto = Cave()
  >>> grotto
  <dolmen.content.tests.overview.base.Cave object at ...>

  >>> icecave = Cave(title=u'An icy cave')
  >>> icecave.title
  u'An icy cave'

  >>> quiver = HunterQuiver()
  >>> quiver
  <dolmen.content.tests.overview.base.HunterQuiver object at ...>

  >>> grokquiver = HunterQuiver(title=u'A mammoth hunter quiver')
  >>> grokquiver.title
  u'A mammoth hunter quiver'

"""



# -*- coding: utf-8 -*-

import martian
from dolmen.content.interfaces import IFactory
from zope.interface import classImplements
from zope.interface.interfaces import IInterface
from zope.interface.advice import addClassAdvisor
from zope.schema.fieldproperty import FieldProperty
from zope.schema import getFieldsInOrder
from zope.schema.interfaces import IField
from zope.interface.interface import InterfaceClass


class Fields(object):
    def __init__(self, *ifaces):
        fields = []
        for iface in ifaces:
            if isinstance(iface, InterfaceClass):
                for name, field in getFieldsInOrder(iface):
                    fields.append((name, field))
            elif IField.providedBy(iface):
                name = iface.__name__
                if not name:
                    raise ValueError(
                        "Field has no name")
                fields.append((name, iface))

        seq = []
        byname = {}
        for name, field in fields:
            name = field.__name__
            if name in byname:
                raise ValueError("Duplicate name", name)
            seq.append(field)
            byname[name] = field

        self.__Fields_seq__ = seq
        self.__Fields_byname__ = byname

    def __len__(self):
        return len(self.__Fields_seq__)

    def __iter__(self):
        return iter(self.__Fields_seq__)

    def __getitem__(self, name):
        return self.__Fields_byname__[name]

    def get(self, name, default=None):
        return self.__Fields_byname__.get(name, default)


def validateSchema(directive, *ifaces):
    for iface in ifaces:
        if not IInterface.providedBy(iface):
            raise martian.error.GrokImportError(
                "%s directive can only use interface classes. "
                "%s is not an interface class. " % (directive.name, iface))


def verifyFactory(directive, factory):
    if factory and not IFactory.implementedBy(factory):
        raise martian.error.GrokImportError(
            "%s directive can only use classes that implement IFactory. "
            "%s is not a valid factory. " % (directive.name, factory))


class schema(martian.Directive):
    scope = martian.CLASS
    store = martian.ONCE
    validate = validateSchema

    def factory(self, *schemas):
        addClassAdvisor(_schema_advice, depth=3)
        return list(schemas)


def _schema_advice(cls):
    interfaces = schema.bind().get(cls)
    
    formfields = Fields(*interfaces)
    for field in formfields:
        fname = field.__name__
        if not hasattr(cls, fname):
            setattr(cls, fname, FieldProperty(field))

    classImplements(cls, *interfaces)
    return cls


class factory(martian.Directive):
    scope = martian.CLASS_OR_MODULE
    store = martian.ONCE
    validate = verifyFactory


class nofactory(martian.MarkerDirective):
    scope = martian.CLASS
    store = martian.ONCE_NOBASE



# -*- coding: utf-8 -*-

import zope.component.interfaces
from dolmen.field import GlobalClass
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface, Attribute
from zope.schema import TextLine

_ = MessageFactory('zope')


class IContent(Interface):
    """Marker interface for contentish dolmen objects.
    """
    __content_type__ = Attribute(
        """Type of the object. Usually set by the grokker, using the
        grok.name directive.""")


class IContainer(IContent):
    """Marker interface for folderish dolmen objects.
    """


class IOrderedContainer(IContainer):
    """Marker interface for folderish dolmen objects with ordered keys.
    """


class IFactory(zope.component.interfaces.IFactory):
    """A factory for a dolmen content type.
    """
    factory = GlobalClass(
        required=True,
        title=u"Class used as a factory",
        schema=IContent)

    addform = TextLine(
        required=True,
        title=u"Name of the add form",
        default=u"dolmen.add")

    name = TextLine(
        required=True,
        title=u"Name of the factored content",
        default=u"")

    title = TextLine(
        required=False,
        title=u"Title of the factored content",
        default=u"")

    description = TextLine(
        required=False,
        title=u"Description of the factored content",
        default=u"")

    def getSchema():
        """Returns a list of interfaces representing the schema
        of the factored item. The returned interfaces must be provided
        by the instance of the object issued from the factory.
        """


__all__ = ('IContent', 'IContainer', 'IOrderedContainer', 'IFactory')



"""
Herited schemas example
=======================

  >>> gilgamesh = MesopotamianGod()
  >>> gilgamesh.homecity
  u'Babylon'

  >>> dolmen.schema.bind().get(gilgamesh)
  [<InterfaceClass ...IMythologicalHero>]
  
"""
import dolmen.content as dolmen
from grokcore.component import baseclass
from grokcore.content import Container
from zope.interface import Interface
from zope.schema import TextLine


class IMythologicalHero(Interface):
    """A hero that transcended History.
    """
    homecity = TextLine(
        title=u"Name of the home city of the Hero",
        required=True,
        default=u"Babylon")


class Hero(dolmen.Content):
    baseclass()
    dolmen.schema(IMythologicalHero)


class AssyrianHero(Hero):
    baseclass()


class MesopotamianGod(Container, AssyrianHero):
    pass



"""
To Grok or not to grok
======================

The use of the `schema` directive works without any grokking.
A simple example :

  >>> from dolmen.content import schema
  >>> from zope.interface import Interface
  >>> from zope.schema import TextLine, Int

  >>> class IMySchema(Interface):
  ...   mystr = TextLine(title=u"A string", default=u"Boo")
  ...   number = Int(title=u"A simple number", default=1)

  >>> class Content(object):
  ...   schema(IMySchema)
  
  >>> IMySchema.implementedBy(Content)
  True

  >>> obj = Content()
  >>> obj.mystr
  u'Boo'
  >>> obj.number
  1
  
"""



#-*- coding: utf-8 -*-

import zope.component
from zope.component.interfaces import IComponentLookup
from zope.component.testlayer import ZCMLFileLayer
from zope.container.interfaces import ISimpleReadContainer
from zope.container.traversal import ContainerTraversable
from zope.interface import Interface
from zope.site.folder import rootFolder
from zope.site.site import LocalSiteManager, SiteManagerAdapter
from zope.traversing.interfaces import ITraversable
from zope.traversing.testing import setUp


def siteSetUp(test):

    zope.component.hooks.setHooks()

    # Set up site manager adapter
    zope.component.provideAdapter(
        SiteManagerAdapter, (Interface,), IComponentLookup)

    # Set up traversal
    setUp()
    zope.component.provideAdapter(
        ContainerTraversable, (ISimpleReadContainer,), ITraversable)

    # Set up site
    site = rootFolder()
    site.setSiteManager(LocalSiteManager(site))
    zope.component.hooks.setSite(site)

    return site


def siteTearDown(test):
    zope.component.hooks.resetHooks()
    zope.component.hooks.setSite()


class DolmenContentLayer(ZCMLFileLayer):
    pass



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.content'
version = '0.7.0'
readme = open(join('src', 'dolmen', 'content', 'README.txt')).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'dolmen.field >= 0.3',
    'grokcore.component >= 2.2',
    'grokcore.content',
    'grokcore.security',
    'martian >= 0.12',
    'setuptools',
    'zope.component',
    'zope.i18nmessageid',
    'zope.interface',
    'zope.schema',
    ]

tests_require = [
    'zope.container',
    'zope.site',
    'zope.traversing',
    'zope.location',
    ]

setup(name = name,
      version = version,
      description = 'Dolmen content type framework',
      long_description = readme + '\n\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen Content',
      author = 'Souheil Chelfouh',
      author_email = 'souheil@chelfouh.com',
      url = 'http://gitweb.dolmen-project.org/',
      download_url = '',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      classifiers = [
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)



"""
One schema example
==================

Conan is a simple Barbarian. Thinking is an everyday challenge for him.
While he's not punching camels, he's resting or drinking in a tavern.
Therefore, only one schema can define him pretty good :

  >>> conan = Barbarian()
  >>> IBarbarian.providedBy(conan)
  True
  >>> dolmen.IContent.providedBy(conan)
  True
 
Of course, our Barbarian is an IBarbarian but still a IContent.
It still has the title and __content_type__ attributes. It also has the
attributes defined in the IBarbarian interface, set by default.

  >>> conan.nickname
  u'The Barbarian'
  >>> conan.kills
  100

"""

import dolmen.content as dolmen
from zope.interface import Interface
from zope.schema import TextLine, Int

    
class IBarbarian(Interface):
    """A barbarian. Usually only wearing a leather underpants.
    """
    kills = Int(
        title=u"Kills !",
        default=100)

    nickname = TextLine(
        title=u"Nickname",
        default=u"The Barbarian")


class Barbarian(dolmen.Content):
    """A barbarian content
    """
    dolmen.name('male')
    dolmen.schema(IBarbarian)



"""
No factories
============

Sometimes, it makes no sense to have factories for objects, for they are not
destined to be used as a content type. In these case, having a factory is
useless and confusing. Therefore, a nofactory directive allows you to
explicitly mark your component as non-eligible for an auto factory grokking,
even if a factory directive is set on the class.

  >>> from zope.component import queryUtility
  >>> factory = queryUtility(dolmen.IFactory, 'horror')
  >>> factory is None
  True

"""

import dolmen.content as dolmen
from zope.schema import Bool
from zope.interface import Interface


class ISomethingThatShouldntBe(Interface):
    """A cyclopean, undescriptable and non euclidean horror.
    """
    sanity_failure = Bool(
        title=u"Seeing this makes you unsane.",
        default=True)


class SunkenTemple(dolmen.Factory):
    dolmen.name('horror')


class Nyarlathotep(dolmen.Content):
    """A crawling chaos
    """
    dolmen.name(u'Impossible to name.')
    dolmen.schema(ISomethingThatShouldntBe)

    # This is contradictory
    dolmen.nofactory()
    dolmen.factory(SunkenTemple)



"""
Auto factories
==============

dolmen.content provides a generic factory for each content type
(see 'simple.py' for more details). It is possible to define a
custom factory by adding a class inheriting from dolmen.Factory
in your module or by using the 'factory' directive. The automatic
binding between the content type and the factory works only in the case
we have 1 context and 1 factory in the same module.

For this test, we have a type of object : ICake. We have an implementation :
SweetRoll. And to finish, we have the person that bakes the cake, Joe, the
Factory. In this file, we have only one IContext and only one Factory class.
Therefore, the link between the two will be done automatically :

  >>> from zope.component import getUtility, queryUtility
  >>> factory = getUtility(dolmen.IFactory, 'Joe')
  >>> isinstance(factory, Baker)
  True
  >>> factory.description
  u'Sweet rolls baker since 1884'
  >>> factory.getSchema()
  [<InterfaceClass dolmen.content.tests.factory.auto_factory.ICake>]

If binded automatically, the generic factory is not registered.

  >>> auto_factory = queryUtility(dolmen.IFactory,
  ...                'dolmen.content.tests.factory.auto_factory.SweetRoll')
  >>> auto_factory is None
  True

We now have a useable factory. We can generate SweetRolls on demand.

  >>> cake = factory()
  >>> ICake.providedBy(cake)
  True

  >>> isinstance(cake, SweetRoll)
  True

  >>> cake.__content_type__
  u'Joes rolled delicacies'

  >>> cake.ingredients
  ['strawberry jam', 'flour', 'eggs']

"""

import dolmen.content as dolmen
from zope.schema import List
from zope.interface import Interface


class ICake(Interface):
    """A sweet eatable object.
    """
    ingredients = List(
        title=u"The ingredients needed to bake the cake.",
        required=True,
        default=[])


class SweetRoll(dolmen.Content):
    """A rolled cake with strawberry jam in it.
    """
    dolmen.schema(ICake)
    dolmen.name(u'Joes rolled delicacies')
    ingredients = ['strawberry jam', 'flour', 'eggs']


class Baker(dolmen.Factory):
    dolmen.name('Joe')
    title = u"Welcome at Joe's bakery"
    description = u"Sweet rolls baker since 1884"
